/*
  Copyright 2003 John Knottenbelt
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
 
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
*/

#include "faketunermanager.h"
#include "faketuner.h"

FakeTunerManager::FakeTunerManager(const TunerManager &tm)
{
  std::map<const Tuner *, Tuner *> subst;

  // Convert the tuner list

  for (TunerList::const_iterator i = tm.tuners.begin(); i != tm.tuners.end(); i++) {
    Tuner *oldTuner = *i;
    FakeTuner *fakeTuner = new FakeTuner(*oldTuner);

    tuners.push_back(fakeTuner);
    subst[oldTuner] = fakeTuner;
  }

  // Convert the type tuner map

  for (TypeTunerMap::const_iterator i = tm.typeTunerMap.begin(); i != tm.typeTunerMap.end(); i++) {
    const std::string &type = i->first;
    const TunerList *oldTunerList = i->second;

    TunerList *fakeTunerList = new TunerList;
    for (TunerList::const_iterator j = oldTunerList->begin(); j != oldTunerList->end(); j++) {
      Tuner *oldTuner = *j;
      fakeTunerList->push_back(subst[oldTuner]);
    }

    typeTunerMap[type] = fakeTunerList;
  }

  // Convert the tuner channel map

  for (TunerChannelMap::const_iterator i = tm.tunerChannelMap.begin(); i != tm.tunerChannelMap.end(); i++) {
    const Tuner *oldTuner = i->first;
    ChannelMap *cm = i->second;

    // NB channel map is aliased here
    // So we shouldn't delete it.
    // (Possibilities are:
    //    - ref count on channel maps
    //    - copy the channel map)

    tunerChannelMap[subst[oldTuner]] = cm;
  }
}
