// -*- c++ -*-
/*
  Copyright 2003 John Knottenbelt
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
 
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
*/

#if !defined __TUNER_H
#define __TUNER_H

#include "source.h"
#include "select.h"
#include "demuxer.h"

#include <string>
#include <map>
#include <ctime>

class TuneParams;
class Demuxer;
class PSFilter;

class Tuner : public Source, public Selectable {
public:
  Tuner( const std::string &frontendDevice );
  virtual ~Tuner();

  const TuneParams *getTuneParams() const { return tuneParams; };
  void setTuneParams(const TuneParams *tuneParams);

  // Returns the corresponding demuxer device
  // to the frontendDevice
  std::string getDemuxerDevice() const { return demuxDevice; };

  virtual void addSelectFDs(Select &) const;
  virtual bool isReady(const Select &) const;

  Source *getDemuxer(int pid, bool convertToPS, Demuxer::PESType pesType) const;

  int getMaxNumSubscribers() const { return 8; }
  virtual bool subscribe(Sink *s);
  virtual void unsubscribe(Sink *s);

  // Process the data from the dvr device
  void process();

protected:
  virtual Demuxer *makeNewDemuxer(int pid, Demuxer::PESType pesType) const;
  virtual bool openFrontend();
  virtual void closeFrontend();
  virtual bool openDvr();
  virtual void closeDvr();
  virtual bool tune();

private:
  void requestCloseFrontend();
  unsigned closeSecondsRemaining() const;

private:
  int numReaders;
  int frontendFD, dvrFD;
  const TuneParams *tuneParams, *oldTuneParams;
  std::string frontendDevice, demuxDevice, dvrDevice;
  
  enum { Open, Closed, Closing } state;
  time_t closeRequestedTime;
  
  // Keep a track of all the demuxers ever created.
  // The Tuner owns the demuxers.

  typedef std::map <int, Demuxer *> PIDMap;
  typedef std::map <const TuneParams *, PIDMap> DemuxerMap;
  typedef std::map <Demuxer *, PSFilter *> PSFilterMap;

  mutable DemuxerMap demuxerMap;
  mutable PSFilterMap psfilterMap;

};

#endif // __TUNER_H
